@@ -8,8 +8,6 @@ if len(sys.argv) > 1:
 else:
     filename = "sample.rtf"
 
-doc = Rtf15Reader.read(open(filename))
+doc = Rtf15Reader.read(open(filename, "rb"))
 
 print XHTMLWriter.write(doc, pretty=True).read()
-
-
