@@ -77,18 +77,19 @@ class BackslashEscape(Exception):
 class Rtf15Reader(PythReader):
 
     @classmethod
-    def read(self, source, errors='strict'):
+    def read(self, source, errors='strict', clean_paragraphs=True):
         """
         source: A list of P objects.
         """
 
-        reader = Rtf15Reader(source, errors)
+        reader = Rtf15Reader(source, errors, clean_paragraphs)
         return reader.go()
 
 
-    def __init__(self, source, errors='strict'):
+    def __init__(self, source, errors='strict', clean_paragraphs=True):
         self.source = source
         self.errors = errors
+        self.clean_paragraphs = clean_paragraphs
         self.document = document.Document
 
 
@@ -193,7 +194,7 @@ class Rtf15Reader(PythReader):
     def build(self):
         doc = document.Document()
 
-        ctx = DocBuilder(doc)
+        ctx = DocBuilder(doc, self.clean_paragraphs)
 
         for bit in self.group.flatten():
             typeName = type(bit).__name__
@@ -207,22 +208,31 @@ class Rtf15Reader(PythReader):
 
 class DocBuilder(object):
 
-    def __init__(self, doc):
+    def __init__(self, doc, clean_paragraphs=True):
         self.run = []
         self.propStack = [{}]
         self.block = None
 
+        self.isImage = False
         self.listLevel = None
         self.listStack = [doc]
 
+        self.clean_paragraphs = clean_paragraphs
+
 
     def flushRun(self):
         if self.block is None:
             self.block = document.Paragraph()
-
-        self.block.content.append(
-            document.Text(self.propStack[-1].copy(),
-                          [u"".join(self.run)]))
+        
+        if self.isImage:
+            self.block.content.append(
+                document.Image(self.propStack[-1].copy(),
+                               [str("".join(self.run))]))
+            self.isImage = False
+        else:
+            self.block.content.append(
+                document.Text(self.propStack[-1].copy(),
+                              [u"".join(self.run)]))
 
         self.run[:] = []
 
@@ -239,6 +249,9 @@ class DocBuilder(object):
             self.block = None
             return
 
+        if not self.clean_paragraphs:
+            return
+
         joinedRuns = []
         hasContent = False
 
@@ -307,7 +320,10 @@ class DocBuilder(object):
             self.listStack[-1].append(l)
 
         self.block = None
-
+    
+    def handle_Pict(self, pict):
+        self.flushRun()
+        self.isImage = True
 
     def handle_Reset(self, _):
         self.flushRun()
@@ -328,6 +344,16 @@ class DocBuilder(object):
             if marker.name in self.propStack[-1]:
                 del self.propStack[-1][marker.name]
 
+    def handle_ImageMarker(self, marker):
+        if marker.val:
+            self.propStack[-1][marker.name] = marker.val
+        else:
+            if marker.name in self.propStack[-1]:
+                # Is there any toggle that is applied to images?
+                del self.propStack[-1][marker.name]
+            else:
+                self.propStack[-1][marker.name] = True
+    
 
 
 class Group(object):
@@ -346,6 +372,7 @@ class Group(object):
         self.specialMeaning = None
         self.skip = False
         self.url = None
+        self.image = None
         self.currentParaTag = None
         self.destination = False
 
@@ -355,10 +382,17 @@ class Group(object):
 
 
     def handle(self, control, digits):
-
         if control == '*':
             self.destination = True
             return
+        
+        if self.image and control in ['emfblip', 'pngblip', 'jpegblip', 'macpict', 'pmmetafile', 'wmetafile', 
+                                      'dibitmap', 'wbitmap', 'wbmbitspixel', 'wbmplanes', 'wbmwidthbytes', 
+                                      'picw', 'pich', 'picwgoal', 'pichgoal', 'picscalex', 'picscaley', 
+                                      'picscaled', 'piccropt', 'piccropb', 'piccropr', 'piccropl', 'picbmp', 
+                                      'picbpp', 'bin', 'blipupi', 'blipuid', 'bliptag', 'wbitmap']:
+            self.content.append(ImageMarker(control, digits))
+            return
 
         handler = getattr(self, 'handle_%s' % control, None)
         if handler is None:
@@ -452,7 +486,14 @@ class Group(object):
             self.fontNum = int(fontNum)
             self._setFontCharset()
         elif self.charsetTable is not None:
-            self.charset = self.charsetTable[int(fontNum)]
+            try:
+                self.charset = self.charsetTable[int(fontNum)]
+            except KeyError:
+                # fontNum not found in charsetTable, ignore if requested
+                if self.reader.errors == 'ignore':
+                    pass
+                else:
+                    raise
 
     def handle_fcharset(self, charsetNum):
         if 'FONT_TABLE' in (self.parent.specialMeaning, self.specialMeaning):
@@ -490,7 +531,7 @@ class Group(object):
 
     def handle_u(self, codepoint):
         codepoint = int(codepoint)
-        try: 
+        try:
             char = unichr(codepoint)
         except ValueError:
             if self.reader.errors == 'replace':
@@ -550,6 +591,11 @@ class Group(object):
     def handle_super(self):
         self.content.append(ReadableMarker("super", True))
 
+    #Turns off superscripting or subscripting
+    def handle_nosupersub(self):
+        self.content.append(ReadableMarker("sub", False))
+        self.content.append(ReadableMarker("super", False))
+
     def handle_dn(self, amount):
         self.content.append(ReadableMarker("sub", True))
 
@@ -574,7 +620,20 @@ class Group(object):
     def handle_rdblquote(self):
         self.content.append(u'\u201D')
 
+    def handle_tab(self):
+        self.content.append(u'\t')
 
+    def handle_trowd(self):
+        self.content.append(u'\n')
+        
+    #Handle the image tag
+    def handle_pict(self):
+        p = Pict()
+        self.content.append(p)
+        self.image = p
+        #Remove the destination control group of the parent, so that the image is preserved
+        self.parent.destination = False
+    
     def handle_field(self):
         def finalize():
             if len(self.content) != 2:
@@ -658,7 +717,16 @@ class ReadableMarker(object):
         else:
             return "!%s::%s!" % (self.name, self.val)
 
+class ImageMarker(ReadableMarker):
+    pass
 
+class Pict(ImageMarker):
+    def __init__(self):
+        ImageMarker.__init__(self, "Pict")
+
+    def __repr__(self):
+        return "!Image!"
+            
 class Para(ReadableMarker):
     listLevel = None
 
